@@ -56,6 +56,18 @@ class MVPoly(object) :
 
         return np.dot(sgn, w)
 
+    def norm(self, order) :
+        """
+        Returns the *norm* of the coefficients.  The *order*
+        can be a float (for the p-norm) or any other value
+        accepted by the :meth:`numpy.linalg.norm` method. 
+        """
+        v = np.array([val for idx, val in self.nonzero])
+        if v.size > 0 :
+            return np.linalg.norm(v, order);
+        else :
+            return 0
+
     def maxmodnd(self, epsilon = 1e-12, i0 = None) :
         """
         The maximum modulus of a complex *n*-variate polynomial on 
@@ -198,3 +210,31 @@ class MVPoly(object) :
             return self.compose(*args)
         else :
             return self.eval(*args)
+
+
+    @classmethod
+    def berenstein(cls, i, n, **kwargs) :
+        """
+        Return the output of :meth:`mvpoly.dict.MVPolyDict.berenstein` 
+        converted to other polynomial subclasess.
+        """
+        from mvpoly.dict import MVPolyDict
+        return cls(MVPolyDict.berenstein(i, n, **kwargs), **kwargs)
+
+    @classmethod
+    def lehmer(cls, **kwargs) :
+        """
+        Return the output of :meth:`mvpoly.cube.MVPolyCube.lehmer` 
+        converted to other polynomial subclasess.
+        """
+        from mvpoly.cube import MVPolyCube
+        return cls(MVPolyCube.lehmer(**kwargs), **kwargs)
+
+    @classmethod
+    def rudin_shapiro(cls, n, **kwargs) :
+        """
+        Return the output of :meth:`mvpoly.cube.MVPolyCube.rudin_shapiro` 
+        converted to other polynomial subclasess.
+        """
+        from mvpoly.cube import MVPolyCube
+        return cls(MVPolyCube.rudin_shapiro(n, **kwargs), **kwargs)
