@@ -16,6 +16,7 @@ import mvpoly.util.common
 
 import numbers
 import numpy as np
+import scipy as sp
 import warnings
 
 class MVPolyDict(mvpoly.base.MVPoly) :
@@ -38,86 +39,6 @@ class MVPolyDict(mvpoly.base.MVPoly) :
                 coef[k] = self.dtype.type(v)
         self._coef = coef
 
-
-    @classmethod
-    def init_from_nonzero(self, p, dtype) :
-        """
-        Create a polynomial from one of any subclass; this is used 
-        internally by the 
-        :py:meth:`~mvpoly.cube.MVPolyDict.__init__` constructor, but
-        may be useful for direct application.
-        """
-        q = self.zero(dtype = dtype)
-        for idx, val in p.nonzero :
-            q[idx] = val
-        return q
-
-
-    @classmethod
-    def monomials(self, n, **kwd) :
-        """
-        Return a *n*-tuple of each of the monomials (*x*, *y*, ..) 
-        of an *n*-variate system.
-        """
-        return [self.monomial(i, n, **kwd) for i in range(n)]
-
-
-    @classmethod
-    def _numpy_scalar(self, v, **kwd) :
-        """
-        Return a :class:`numpy` scalar which, if the optional
-        argument contains a ``dtype`` entry, will be of that 
-        *dtype*.
-        """
-        dtype = kwd.get('dtype', False) 
-        if dtype :
-            return np.dtype(dtype).type(v)
-        else :
-            return np.int_(v)
-
-
-    @classmethod
-    def monomial(self, i, n, **kwd) :
-        """
-        Return the *i*-th monomial of an *n*-variate system,
-        (*i* = 0, ..., *n*-1).
-        """
-        m = MVPolyDictMonomial({i:1})
-        d = { m.key : self._numpy_scalar(1, **kwd) }
-        return MVPolyDict(d, **kwd)
-
-
-    @classmethod
-    def zero(self, **kwd) :
-        """
-        Return the zero polynomial. 
-        """
-        return MVPolyDict(**kwd)
-
-
-    @classmethod
-    def one(self, **kwd) :
-        """
-        Return the unit (1) polynomial. 
-        """
-        m = MVPolyDictMonomial({})
-        d = { m.key: self._numpy_scalar(1, **kwd) }
-        return MVPolyDict(d, **kwd)
-
-
-    @property
-    def coef(self) :
-        """
-        Return the dictionary of coefficients. 
-        """
-        return self._coef
-
-
-    @coef.setter
-    def coef(self, value) :
-        self._coef = value
-
-
     def __setitem__(self, index, value) :
         """
         The setter method, which sets coefficient of the
@@ -170,6 +91,15 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         return self.__add__(other)
 
 
+    def __neg__(self) :
+        """
+        Negation of a polynomial, return the polynomial with negated 
+        coefficients.
+        """
+        d = mvpoly.util.dict.negate(self.coef)
+        return MVPolyDict(d, dtype = self.dtype)
+
+
     def __mul__(self, other) :
         """
         Product of polynomials, or of a polynomial and a number.
@@ -209,21 +139,124 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         return self.__mul__(other)
 
 
-    def __neg__(self) :
-        """
-        Negation of a polynomial, return the polynomial with negated 
-        coefficients.
-        """
-        d = mvpoly.util.dict.negate(self.coef)
-        return MVPolyDict(d, dtype = self.dtype)
-
-
     def __eq__(self, other) :
         """
         Equality of polynomials. 
         """
         return not any((self - other).coef)
 
+    
+    def astype(self, dtype) :
+        """
+        Return a polynomial using the specified *dtype* for the
+        coefficients.
+        """
+        return MVPolyDict.init_from_nonzero(self, dtype=dtype)
+
+
+    @property
+    def coef(self) :
+        """
+        Return the dictionary of coefficients. 
+        """
+        return self._coef
+
+    @coef.setter
+    def coef(self, value) :
+        self._coef = value
+
+
+    def compose(self, *args) :
+        """
+        Compose polynomials. The arguments, which should be
+        :class:`MVPolyDict` polynomials, are substituted 
+        into the corresponding variables of the polynomial.
+        """
+        return self.rmf_eval(*args);
+
+
+    @property
+    def degrees(self) :
+        """
+        Return the maximal degrees of each of the variables of
+        the polynomial as a tuple of integers
+        """
+        idxs = [ MVPolyDictMonomial(key = key).index for
+                 key in self.coef.keys() ]
+        n = mvpoly.util.common.max_or_default((len(idx) for idx in idxs), 0)
+        idxs = [idx + (0,) * (n - len(idx)) for idx in idxs]
+        if idxs == [] :
+            degs = () 
+        else :
+            degs = tuple(max(idx[i] for idx in idxs) for i in range(n)) 
+        return degs
+
+
+    @property
+    def degree(self) :
+        """
+        The (total, homogeneous) *degree*, the maximal sum of the
+        monomial degrees of monomials with nonzero coefficients. 
+        Returns -1 for the zero polynomial.
+        """
+        degs = [np.sum(MVPolyDictMonomial(key = k).dict.values())
+                for k in self.coef.keys()]
+        if not degs :
+            return -1
+        return max(degs)
+
+
+    def diff(self, *args) :
+        """
+        Differentiate polynomial. The integer arguments
+        should indicate the number to times to differentiate
+        with respect to the corresponding polynomial variable,
+        hence ``p.diff(0,1,1)`` would correspond to 
+        :math:`\partial^2 p / \partial y \partial z`.
+        """
+        def diff2(idx, m0, m0_coef) :
+            m0_dict  = m0.dict
+            m1_dict = {}
+            for i, n in enumerate(idx) :
+                m0i = m0_dict.get(i, 0)
+                if m0i < n :
+                    return (None, 0)
+                if m0i == n :
+                    continue
+                m1_dict[i] = m0i - n
+                for k in range(n) :
+                    m0_coef *= (m0i - k)
+            return (MVPolyDictMonomial(dict = m1_dict), m0_coef)
+
+        d = {}
+        for m0_key, m0_coef in self.coef.iteritems() :
+            m0 = MVPolyDictMonomial(key = m0_key)
+            m1, m1_coef = diff2(args, m0, m0_coef)
+            if m1 is not None :
+                d[m1.key] = m1_coef
+
+        return MVPolyDict(d, dtype = self.dtype)
+
+
+    def eval(self, *args) :
+        """
+        Evaluate the polynomial at the points given by *args*.
+        There should be as many arguments as the polynomial 
+        has variables.  The argument can be numbers, or arrays
+        (all of the same shape).
+        """
+        # if the arguments are lists then we need to convert
+        # them to numpy arrays, since rmf_eval() requires that
+        array_args = [np.array(arg) for arg in args]
+        array_shps = set(arg.shape for arg in array_args)
+        if len(array_shps) != 1 :
+            raise ValueError("eval() with arguments of differing shapes")
+        array_shp = array_shps.pop()
+        if array_shp == (1,) :
+            return self.rmf_eval(*args);
+        else :
+            return self.rmf_eval(*array_args);
+
 
     def rmf_eval(self, *args) :
         """
@@ -278,116 +311,6 @@ class MVPolyDict(mvpoly.base.MVPoly) :
                     k -= 1
         return s[0]
 
-    @property
-    def nonzero(self) :
-        """
-        Returns a list of 2-element tuples, for each the first
-        being a (monomial) index, the second the corresponding 
-        coefficient. The indices do not have trailing zeros,
-        so the return value for the polynomial
-        :math:`p(x,y) = 2 + 3x + 4y` would be
-        ``[((), 2), ((1,), 3), ((0,1), 4)]``.
-        """
-        return [(MVPolyDictMonomial(key = k).index, v) 
-                for k, v in self.coef.iteritems()]
-
-    @property
-    def degrees(self) :
-        """
-        Return the maximal degrees of each of the variables of
-        the polynomial as a tuple of integers
-        """
-        idxs = [ MVPolyDictMonomial(key = key).index for
-                 key in self.coef.keys() ]
-        n = mvpoly.util.common.max_or_default((len(idx) for idx in idxs), 0)
-        idxs = [idx + (0,) * (n - len(idx)) for idx in idxs]
-        if idxs == [] :
-            degs = () 
-        else :
-            degs = tuple(max(idx[i] for idx in idxs) for i in range(n)) 
-        return degs
-
-    @property
-    def degree(self) :
-        """
-        The (total, homogeneous) *degree*, the maximal sum of the
-        monomial degrees of monomials with nonzero coefficients. 
-        Returns -1 for the zero polynomial.
-        """
-        degs = [np.sum(MVPolyDictMonomial(key = k).dict.values())
-                for k in self.coef.keys()]
-        if not degs :
-            return -1
-        return max(degs)
-    
-    def astype(self, dtype) :
-        """
-        Return a polynomial using the specified *dtype* for the
-        coefficients.
-        """
-        return MVPolyDict.init_from_nonzero(self, dtype=dtype)
-
-    def eval(self, *args) :
-        """
-        Evaluate the polynomial at the points given by *args*.
-        There should be as many arguments as the polynomial 
-        has variables.  The argument can be numbers, or arrays
-        (all of the same shape).
-        """
-        # if the arguments are lists then we need to convert
-        # them to numpy arrays, since rmf_eval() requires that
-        array_args = [np.array(arg) for arg in args]
-        array_shps = set(arg.shape for arg in array_args)
-        if len(array_shps) != 1 :
-            raise ValueError("eval() with arguments of differing shapes")
-        array_shp = array_shps.pop()
-        if array_shp == (1,) :
-            return self.rmf_eval(*args);
-        else :
-            return self.rmf_eval(*array_args);
-
-
-    def compose(self, *args) :
-        """
-        Compose polynomials. The arguments, which should be
-        :class:`MVPolyDict` polynomials, are substituted 
-        into the corresponding variables of the polynomial.
-        """
-        return self.rmf_eval(*args);
-
-
-    def diff(self, *args) :
-        """
-        Differentiate polynomial. The integer arguments
-        should indicate the number to times to differentiate
-        with respect to the corresponding polynomial variable,
-        hence ``p.diff(0,1,1)`` would correspond to 
-        :math:`\partial^2 p / \partial y \partial z`.
-        """
-        def diff2(idx, m0, m0_coef) :
-            m0_dict  = m0.dict
-            m1_dict = {}
-            for i, n in enumerate(idx) :
-                m0i = m0_dict.get(i, 0)
-                if m0i < n :
-                    return (None, 0)
-                if m0i == n :
-                    continue
-                m1_dict[i] = m0i - n
-                for k in range(n) :
-                    m0_coef *= (m0i - k)
-            return (MVPolyDictMonomial(dict = m1_dict), m0_coef)
-
-        d = {}
-        for m0_key, m0_coef in self.coef.iteritems() :
-            m0 = MVPolyDictMonomial(key = m0_key)
-            m1, m1_coef = diff2(args, m0, m0_coef)
-            if m1 is not None :
-                d[m1.key] = m1_coef
-
-        return MVPolyDict(d, dtype = self.dtype)
-
-
     def int(self, *args, **kwargs) :
         """
         Indefinite integral of polynomial. The arguments are
@@ -419,6 +342,124 @@ class MVPolyDict(mvpoly.base.MVPoly) :
         return MVPolyDict(d, dtype = dtype)
 
 
+    @property
+    def nonzero(self) :
+        """
+        Returns a list of 2-element tuples, for each the first
+        being a (monomial) index, the second the corresponding 
+        coefficient. The indices do not have trailing zeros,
+        so the return value for the polynomial
+        :math:`p(x,y) = 2 + 3x + 4y` would be
+        ``[((), 2), ((1,), 3), ((0,1), 4)]``.
+        """
+        return [(MVPolyDictMonomial(key = k).index, v) 
+                for k, v in self.coef.iteritems()]
+
+
+    @classmethod
+    def init_from_nonzero(cls, p, dtype) :
+        """
+        Create a polynomial from one of any subclass; this is used 
+        internally by the 
+        :py:meth:`~mvpoly.cube.MVPolyDict.__init__` constructor, but
+        may be useful for direct application.
+        """
+        q = cls.zero(dtype = dtype)
+        for idx, val in p.nonzero :
+            q[idx] = val
+        return q
+
+
+    @classmethod
+    def monomials(cls, n, **kwd) :
+        """
+        Return a *n*-tuple of each of the monomials (*x*, *y*, ..) 
+        of an *n*-variate system.
+        """
+        return [cls.monomial(i, n, **kwd) for i in range(n)]
+
+
+    @classmethod
+    def _numpy_scalar(cls, v, **kwd) :
+        """
+        Return a :class:`numpy` scalar which, if the optional
+        argument contains a ``dtype`` entry, will be of that 
+        *dtype*.
+        """
+        dtype = kwd.get('dtype', False) 
+        if dtype :
+            return np.dtype(dtype).type(v)
+        else :
+            return np.int_(v)
+
+
+    @classmethod
+    def monomial(cls, i, n, **kwd) :
+        """
+        Return the *i*-th monomial of an *n*-variate system,
+        (*i* = 0, ..., *n*-1).
+        """
+        m = MVPolyDictMonomial({i:1})
+        d = { m.key : cls._numpy_scalar(1, **kwd) }
+        return cls(d, **kwd)
+
+
+    @classmethod
+    def zero(cls, **kwd) :
+        """
+        Return the zero polynomial. 
+        """
+        return cls(**kwd)
+
+
+    @classmethod
+    def one(cls, **kwd) :
+        """
+        Return the unit (1) polynomial. 
+        """
+        m = MVPolyDictMonomial({})
+        d = { m.key: cls._numpy_scalar(1, **kwd) }
+        return cls(d, **kwd)
+
+
+    @classmethod
+    def berenstein(cls, i, n, **kwd) :
+        """
+        Return the degree *n*
+        `Berenstein polynomial <http://www.encyclopediaofmath.org/index.php/Bernstein_polynomials>`_ 
+        of the specified index *i*. Both *n* and *i* may
+        be lists or tuples of non-negative numbers of the
+        same size and with *i* dominated by *n*.
+        """
+        def coerce_list(m) :
+            if isinstance(m, basestring) : m = [m]
+            else:
+                try: iter(m)
+                except TypeError: m = [m]
+                else: m = list(m)
+            return m
+
+        i = coerce_list(i)
+        n = coerce_list(n)
+        if len(i) != len(n) :
+            raise ValueError("index and degree not of same length")
+        inpairs = zip(i, n)
+        nvar    = len(inpairs)
+        if any(ik > nk for ik, nk in inpairs) :
+            raise ValueError("bad index %s for degree %s" % (repr(i), repr(n)))
+
+        q  = cls.one(**kwd)
+        xs = cls.monomials(nvar, **kwd)
+
+        for k in range(nvar) :
+            ik, nk = inpairs[k]
+            xk = xs[k]
+            C = mvpoly.util.common.binom(nk, ik)
+            q = q * C * xk**ik * (1 - xk)**(nk-ik) 
+
+        return q
+
+
 class EvalNode(object) :
     """
     Helper class for the :meth:`__call__` method.
@@ -472,6 +513,22 @@ class MVPolyDictMonomial(object) :
         else :
             self._dict = dict
 
+    def __mul__(self, other) :
+        """
+        Return the product of monomials.
+        """
+        d = mvpoly.util.dict.merge_dicts(self.dict, 
+                                         other.dict, 
+                                         lambda x, y : x+y) 
+        return MVPolyDictMonomial(dict = d)
+
+    @property
+    def degree(self) :
+        """
+        The degree of the monomial: the sum of exponents.
+        """
+        return sum(self.dict.itervalues())
+
     @property
     def dict(self) :
         """
@@ -483,18 +540,6 @@ class MVPolyDictMonomial(object) :
     def dict(self, d) :
         self._dict = d
 
-    @property
-    def key(self) :
-        """
-        A hashable representation (as a tuple of pairs) of the
-        dictionary.
-        """
-        return tuple((k, self.dict[k]) for k in sorted(self.dict.keys()))
-
-    @key.setter
-    def key(self, kvs) :
-        self._dict = {k: v for (k, v) in kvs}
-
     @property
     def index(self) :
         """
@@ -521,6 +566,18 @@ class MVPolyDictMonomial(object) :
         """
         return tuple(self.dict.get(k, 0) for k in range(n))
 
+    @property
+    def key(self) :
+        """
+        A hashable representation (as a tuple of pairs) of the
+        dictionary.
+        """
+        return tuple((k, self.dict[k]) for k in sorted(self.dict.keys()))
+
+    @key.setter
+    def key(self, kvs) :
+        self._dict = {k: v for (k, v) in kvs}
+
     @property
     def occurences(self) :
         """
@@ -532,20 +589,4 @@ class MVPolyDictMonomial(object) :
         monomial.
         """
         return tuple(i for i, e in enumerate(self.index) for _ in range(e))
-
-    @property
-    def degree(self) :
-        """
-        The degree of the monomial: the sum of exponents.
-        """
-        return sum(self.dict.itervalues())
-
-    def __mul__(self, other) :
-        """
-        Return the product of monomials.
-        """
-        d = mvpoly.util.dict.merge_dicts(self.dict, 
-                                         other.dict, 
-                                         lambda x, y : x+y) 
-        return MVPolyDictMonomial(dict = d)
         
