@@ -630,3 +630,111 @@ class TestMVPolyMaxmodND(unittest.TestCase) :
             self.assertTrue(np.abs(exp - obt) < eps*exp, 
                             "bad kaijser-varopoulos maxmodnd %s" % (repr(obt)))
 
+class TestMVPolyNorm(unittest.TestCase) :
+
+    def test_norm_zero(self) :
+        for C in classes :
+            p = C.zero(dtype=int)
+            self.assertTrue(p.norm(1) == 0, "1-norm")
+            self.assertTrue(p.norm(2) == 0, "2-norm")
+            self.assertTrue(p.norm(np.inf) == 0, "inf-norm")
+
+    def test_norm_simple(self) :
+        for C in classes :
+            x, y = C.monomials(2, dtype=int)
+            p = x**2 - y**2
+            self.assertTrue(p.norm(1) == 2, "1-norm")
+            self.assertTrue(abs(p.norm(2) -  np.sqrt(2)) < 1e-10, "2-norm")
+            self.assertTrue(p.norm(np.inf) == 1, "inf-norm")
+            
+
+class TestMVPolyRudinShapiro(unittest.TestCase) :
+
+    def test_rudin_shapiro_size(self) :
+        for C in classes :
+            for n in range(3, 6) :
+                p = C.rudin_shapiro(n)
+                self.assertTrue(p.degrees == (2**n-1,), "rudin-shapiro degrees")
+
+    def test_rudin_shapiro_dtype_default(self) :
+        for C in classes :
+            p = C.rudin_shapiro(3)
+            self.assertTrue(p.dtype == int, "rudin-shapiro default dtype")
+
+    def test_rudin_shapiro_dtype_user(self) :
+        for C in classes :
+            p = C.rudin_shapiro(3, dtype=np.complex)
+            self.assertTrue(p.dtype == np.complex, "rudin-shapiro user dtype")
+
+    def test_rudin_shapiro_2(self) :
+        for C in classes :
+            x, = C.monomials(1, dtype = int)
+            p = C.rudin_shapiro(2)
+            q = 1 + x + x**2 - x**3
+            self.assertTrue(p == q, "rudin-shapiro(2)")
+
+    def test_rudin_shapiro_maxmod(self) :
+        # this inequality is what makes these polynomials interesting,
+        # they have rather small maximum modulu
+        for C in classes :
+            eps = 1e-10
+            for m in range(2,7) :
+                p = C.rudin_shapiro(m).astype(np.complex)
+                M, ts, h, neval = p.maxmodnd(epsilon=eps)
+                self.assertTrue(M*(1-eps) < np.sqrt(2**(m+1)), 
+                                "rudin-shapiro maxmod")
+
+    def test_rudin_shapiro_L4_norm(self) :
+        # for norm tests : Littlewood, if n = 2^k - 1, P the Rudin-Shapiro
+        # polynomial order n, then 
+        #
+        #       ||P||_L4^4  =  (1/3)((4n+1)^2 - (-1)^k (n+1))
+        # 
+        # to be done when norm_L(p) is implemented
+        pass
+
+class TestMVPolyLehmer(unittest.TestCase) :
+
+    def test_lehmer_size(self) :
+        for C in classes :
+            p = C.lehmer(dtype=int)
+            self.assertTrue(p.degrees == (10,), "rudin-shapiro degrees")
+
+    def test_lehmer_dtype(self) :
+        for C in classes :
+            for dt in [int, float, np.double] :
+                p = C.lehmer(dtype=dt)
+                self.assertTrue(p.dtype == dt, "rudin-shapiro dtype")
+
+    def test_lehmer_explicit(self) :
+        for C in classes :
+            p = C.lehmer(dtype=int)
+            x, = C.monomials(1, dtype=int)
+            q = x**10 + x**9 - x**7 - x**6 - x**5 - x**4 - x**3 + x + 1
+            self.assertTrue(p == q, "rudin-shapiro explicit")
+            
+
+class TestMVPolyBerenstein(unittest.TestCase) :
+
+    def test_berenstein_size(self) :
+        for C in classes :
+            p = C.berenstein((3, 2), (5, 7), dtype=int)
+            self.assertTrue(p.degrees == (5,7), "berenstein degrees")
+
+    def test_berenstein_badargs(self) :
+        for C in classes :
+            self.assertRaises(ValueError, C.berenstein, (3, 8), (5, 7))
+            self.assertRaises(ValueError, C.berenstein, (1, 2, 3), (5, 7))
+
+    def test_berenstein_explicit_1d(self) :
+        for C in classes :
+            p = C.berenstein(3, 4, dtype=int)
+            x = C.monomial(0, 1, dtype=int)
+            q = 4 * x**3 * (1 - x)
+            self.assertTrue(p == q, "berenstein explicit 1d")
+
+    def test_berenstein_dtype(self) :
+        for C in classes :
+            for dt in [int, float, np.double] :
+                p = C.berenstein((3,2), (5,7), dtype=dt)
+                self.assertTrue(p.dtype == dt, "rudin-shapiro dtype")
