@@ -1,7 +1,16 @@
 import numpy as np
+import scipy as sp
 import mvpoly.util.common
 import unittest
 
+class TestMPUtilCommonBinom(unittest.TestCase) :
+    
+    def test_binom_reference(self) :
+        for n, i in [ (3,2), (5,0), (7, 6) ] :
+            a = mvpoly.util.common.binom(n, i)
+            b = long(round(sp.special.binom(n, i)))
+            self.assertTrue(a == b, "binomial against reference")
+
 class TestMPUtilCommonKronn(unittest.TestCase):
 
     def test_kronn_simple(self) :
